package net.openseed.osaf.docgen;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.dialect.Dialect;
import org.hibernate.dialect.MySQL5Dialect;

import net.openseed.osaf.docgen.writer.DocWriter;
import net.openseed.osaf.docgen.writer.HTMLDocWriter;

public class DomainDocGen {

	@SuppressWarnings("unchecked")
	private List<Class> entityClasses = new ArrayList<Class>();
	
	private String targetFolderName;
	
	private DocWriter docWriter;
	
	private Dialect dialect;

	public DomainDocGen(List<String> entityClassNames, String targetFolderName, DocWriter docWriter, Dialect dialect) {
		initEntityClasses(entityClassNames);
		this.targetFolderName = targetFolderName;
		
		if(docWriter == null) 
			this.docWriter = new HTMLDocWriter();
		else
			this.docWriter = docWriter;
		
		if(dialect == null)
			this.dialect = new MySQL5Dialect();
		else
			this.dialect = dialect;
	}

	public DomainDocGen(List<String> entityClassNames, String targetFolderName, Dialect dialect) {
		this(entityClassNames, targetFolderName, null, dialect);
	}

	public DomainDocGen(List<String> entityClassNames, String targetFolderName) {
		this(entityClassNames, targetFolderName, null, null);
	}
	private void initEntityClasses(List<String> entityClassNames) {
		for(String className : entityClassNames){
			try {
				entityClasses.add(Class.forName(className));
			} catch (ClassNotFoundException e) {
				throw new RuntimeException(e);
			}
		}
	}

	public int getNumberOfEntityClasses() {
		return entityClasses.size();
	}

	public List<Class> getEntityClasses() {
		return entityClasses;
	}

	public String getTargetFolder() {
		return targetFolderName;
	}

	public void createDoc() {
		docWriter.makeIndexPage(targetFolderName, entityClasses);
		for(Class entityClass : entityClasses){
			docWriter.makeEntityPage(targetFolderName, entityClass, dialect);
		}
	}

	public void createTargetFolder() {
		File targetFolder = new File(targetFolderName);
		if(!targetFolder.exists()) targetFolder.mkdirs();
	}
	
	public void run() {
		createTargetFolder();
		createDoc();
		System.out.println("complete");
	}

}
