package net.openseed.osaf.docgen.info;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;

import javax.persistence.CascadeType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

import net.openseed.osaf.util.StringUtils;

public class AssociationInfo {

	String name;
	String associationType;
	String associationClassName;
	String associationCollectionType;

	String cascade;
	String fetch;
	String mappedBy;

	String joinColumn;

	ColumnInfo columnInfo;

	public AssociationInfo(Field f, Class a) {
		name = f.getName();
		associationType = a.getSimpleName();
		Class ac = null;
		if (f.getGenericType() instanceof ParameterizedType) {
			ParameterizedType genericType = (ParameterizedType) f
					.getGenericType();
			ac = (Class) genericType.getActualTypeArguments()[0];
			String typeFullName = genericType.getRawType().toString();
			associationCollectionType = typeFullName.substring(typeFullName
					.indexOf(" ") + 1);
		} else {
			ac = f.getType();
		}
		associationClassName = ac.getSimpleName();
		columnInfo = new ColumnInfo(f);

		for (Annotation anno : f.getAnnotations()) {
			if (anno instanceof OneToOne)
				fillOneToOneInfos((OneToOne) anno);
			if (anno instanceof ManyToOne)
				fillManyToOneInfos((ManyToOne) anno);
			if (anno instanceof ManyToMany)
				fillManyToManyInfos((ManyToMany) anno);
			if (anno instanceof OneToMany)
				fillOneToManyInfos((OneToMany) anno);
			if (anno instanceof JoinColumn)
				fillJoinColumnInfos((JoinColumn) anno);
		}

		fillDefaultValues();

	}

	private void fillDefaultValues() {
		if (StringUtils.isEmpty(joinColumn))
			joinColumn = name + "_id";
		if (StringUtils.isEmpty(associationCollectionType))
			associationCollectionType = "";
		if (StringUtils.isEmpty(cascade))
			cascade = "";
	}

	private void fillJoinColumnInfos(JoinColumn anno) {
		joinColumn = anno.name();
	}

	private void fillOneToManyInfos(OneToMany anno) {
		mappedBy = anno.mappedBy();
		for (CascadeType ct : anno.cascade()) {
			cascade += ct.name() + " ";
			cascade = cascade.substring(4);
		}
		fetch = anno.fetch().name();
	}

	private void fillManyToManyInfos(ManyToMany anno) {
		mappedBy = anno.mappedBy();
		for (CascadeType ct : anno.cascade()) {
			cascade += ct.name() + " ";
		}
		fetch = anno.fetch().name();
	}

	private void fillManyToOneInfos(ManyToOne anno) {
		mappedBy = "";
		for (CascadeType ct : anno.cascade()) {
			cascade += ct.name() + " ";
		}
		fetch = anno.fetch().name();
	}

	private void fillOneToOneInfos(OneToOne anno) {
		mappedBy = anno.mappedBy();
		for (CascadeType ct : anno.cascade()) {
			cascade += ct.name() + " ";
		}
		fetch = anno.fetch().name();
	}

	public String getName() {
		return name;
	}

	public String getAssociationType() {
		return associationType;
	}

	public String getAssociationClassName() {
		return associationClassName;
	}

	public ColumnInfo getColumnInfo() {
		return columnInfo;
	}

	public String getAssociatedURL() {
		return associationClassName + ".html";
	}

	public String getAssociationCollectionType() {
		return associationCollectionType;
	}

	public String getCascade() {
		return cascade;
	}

	public String getJoinColumn() {
		return joinColumn;
	}

	public String getMappedBy() {
		return mappedBy;
	}

	public String getFetch() {
		return fetch;
	}

}
