package net.openseed.osaf.docgen.info;

import static java.sql.Types.DATE;
import static java.sql.Types.DOUBLE;
import static java.sql.Types.INTEGER;
import static java.sql.Types.TIME;
import static java.sql.Types.TIMESTAMP;
import static java.sql.Types.VARCHAR;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;

import javax.persistence.Column;
import javax.persistence.Temporal;

import net.openseed.osaf.util.StringUtils;

import org.hibernate.SessionFactory;
import org.hibernate.annotations.Index;
import org.hibernate.dialect.Dialect;
import org.hibernate.dialect.MySQL5Dialect;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public class FieldInfo {

	String name;
	String type;

	String columnName;
	String isUnique;
	String isNullable;
	String indexName;
	String sqlType;

	String temparalType;

	ColumnInfo columnInfo;
	
	Dialect dialect;
	private int length;
	private int precision;
	private int scale;

	public FieldInfo(Field f, Dialect dialect) {
		this.dialect = dialect;
		name = f.getName();
		type = f.getType().getSimpleName();
		Annotation[] as = f.getAnnotations();
		for (Annotation a : as) {
			if (a instanceof Column)
				fillColumnInfos((Column) a);
			if (a instanceof Temporal)
				fillTemporalInfos((Temporal) a);
			if (a instanceof Index)
				fillIndexInfos((Index) a);
		}
		columnInfo = new ColumnInfo(f);
		fillDefaultValues();
	}

	private void fillDefaultValues() {
		if (StringUtils.isEmpty(indexName))
			indexName = "";
		if(StringUtils.isEmpty(columnName))
			columnName = name;
		if (StringUtils.isEmpty(sqlType)) {
			sqlType = dialect.getTypeName(getSQLType(), length, precision, scale);
		}
	}

	private int getSQLType() {
		if(type.equals("int") || type.equals("INTEGER"))
			return INTEGER;
		if(type.equals("double") || type.equals("DOUBLE"))
			return DOUBLE;
		if(type.equals("Date")){
			if(temparalType.equals("DATE"))
				return DATE;
			if(temparalType.equals("TIMESTAMP"))
				return TIMESTAMP;
			if(temparalType.equals("TIME"))
				return TIME;
		}
		return VARCHAR;
	}

	private SessionFactory getSessionFactory() {
		ApplicationContext ac = new ClassPathXmlApplicationContext("/sessionFactory.xml");
		return (SessionFactory) ac.getBean("sessionFactory");
	}

	private void fillIndexInfos(Index a) {
		indexName = a.name();
	}

	private void fillTemporalInfos(Temporal a) {
		temparalType = a.value().name();
	}

	private void fillColumnInfos(Column column) {
		columnName = column.name();
		isUnique = column.unique() ? "O" : "";
		isNullable = column.nullable() ? "O" : "";
		sqlType = column.columnDefinition();
		length = column.length();
		precision = column.precision();
		scale = column.scale();
	}

	public String getName() {
		return name;
	}

	public String getType() {
		return type;
	}

	public String getColumnName() {
		return columnName;
	}

	public String getIsUnique() {
		return isUnique;
	}

	public String getIsNullable() {
		return isNullable;
	}

	public String getTemparalType() {
		return temparalType;
	}

	public ColumnInfo getColumnInfo() {
		return columnInfo;
	}

	public String getIndexName() {
		return indexName;
	}

	public String getSqlType() {
		return sqlType;
	}

}
