package net.openseed.osaf.docgen.writer;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.dialect.Dialect;

import net.openseed.osaf.docgen.info.AssociationInfo;
import net.openseed.osaf.docgen.info.EntityInfo;
import net.openseed.osaf.docgen.info.FieldInfo;
import sjp.hg.Document;
import sjp.hg.DocumentFactory;
import sjp.hg.Element;

public class HTMLDocWriter implements DocWriter {
	
	public void makeEntityPage(String targetFolderName, Class entityClass, Dialect dialect) {
		File newHTML = new File(targetFolderName, entityClass.getSimpleName() + ".html");
		FileWriter fw = null;
		try {
			newHTML.createNewFile();
			fw = new FileWriter(newHTML);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		
		Document doc = null;
		if(fw != null)
			doc = makeDoc(new EntityInfo(entityClass, dialect));
		
		try {
			fw.write(doc.toString());
			fw.flush();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	private Document makeDoc(EntityInfo entityInfo) {
		Document doc = DocumentFactory.makeDocument();
		
		Element head = doc.getHead();
		head.addElement("title").addText(entityInfo.getEntityName());
		
		Element body = doc.getBody();
		body.addAttribute("style", "font-size:10pt;");
		body.addElement("h3").addText("KOMA Domain Model Class: " + entityInfo.getFullPackageClassName());
		
	    Element linkElt = body.addElement("p").addElement("a");
		linkElt.addText("[go to domain model index]");
	    linkElt.addAttribute("href", "index.html");
	    
	    // =====================================================
	    // Entity Info
	    // =====================================================
	    body.addElement("h4").addText("Entity Overview");
	    Element ul = body.addElement("p").addElement("ul");
	    ul.addElement("li").addText("Ŭ : " + entityInfo.getEntityName());
	    ul.addElement("li").addText("̺ : " + entityInfo.getTableName());
	    ul.addElement("li").addText(" : " + entityInfo.getSequenceName());
	    ul.addElement("li").addText("ε : " + entityInfo.getIndexesBySpace());
	    ul.addElement("li").addText("̹Ʈ Optimistic Locking ʵ(version): " + entityInfo.getVersion());
	    
	    // =====================================================
	    // PK Info
	    // =====================================================
	    body.addElement("h4").addText("PK");
	    Element tablePk = body.addElement("p").addElement("table");
	    tablePk.addAttribute("cellpadding", "3");
	    tablePk.addAttribute("cellspacing", "3");
	    tablePk.addAttribute("border", "1");
	    tablePk.addAttribute("style", "border-collapse:collapse;border-color:#999999;font-size:10pt;");
	    Element thtrPk = tablePk.addElement("tr");
	    thtrPk.addAttribute("bgColor", "#a4db42");
	    thtrPk.addElement("th").addText("ʵ");
	    thtrPk.addElement("th").addText("ʵŸ");
	    thtrPk.addElement("th").addText("Generation Type");
	    thtrPk.addElement("th").addText("Generator");
	    Element tdtrPk = tablePk.addElement("tr");
	    tdtrPk.addElement("td").addText(entityInfo.getPkInfo().getName());
	    tdtrPk.addElement("td").addText(entityInfo.getPkInfo().getType());
	    tdtrPk.addElement("td").addText(entityInfo.getPkInfo().getGenerationType());
	    tdtrPk.addElement("td").addText(entityInfo.getPkInfo().getGeneratorName());
	    
	    // =====================================================
	    // Association Info
	    // =====================================================
		if(entityInfo.getAinfos().size() > 0){
			body.addElement("h4").addText("Associations");
			Element tableAssociation = body.addElement("p").addElement("table");
			tableAssociation.addAttribute("cellpadding", "3");
			tableAssociation.addAttribute("cellspacing", "3");
			tableAssociation.addAttribute("border", "1");
			tableAssociation.addAttribute("style", "border-collapse:collapse;border-color:#999999;font-size:10pt;");
			Element thtrAssociation = tableAssociation.addElement("tr");
			thtrAssociation.addAttribute("bgColor", "#a4db42");
			thtrAssociation.addElement("th").addText("ʵ");
			thtrAssociation.addElement("th").addText("ʵŸ");
			thtrAssociation.addElement("th").addText("");
			thtrAssociation.addElement("th").addText("÷");
			thtrAssociation.addElement("th").addText("Cascade");
			thtrAssociation.addElement("th").addText("Fetch");
			thtrAssociation.addElement("th").addText("MappedBy");
			thtrAssociation.addElement("th").addText("JoinColumn");
			for(AssociationInfo ai : entityInfo.getAinfos()){
				Element tdtrAssociation = tableAssociation.addElement("tr");
				tdtrAssociation.addElement("td").addText(ai.getName());
		    	tdtrAssociation.addElement("td").addText(ai.getAssociationType());
		    	Element linkTd = tdtrAssociation.addElement("td").addElement("a");
		    	linkTd.addText(ai.getAssociationClassName());
		    	linkTd.addAttribute("href", ai.getAssociatedURL());
		    	tdtrAssociation.addElement("td").addText(ai.getAssociationCollectionType());
		    	tdtrAssociation.addElement("td").addText(ai.getCascade());
		    	tdtrAssociation.addElement("td").addText(ai.getFetch());
		    	tdtrAssociation.addElement("td").addText(ai.getMappedBy());
		    	tdtrAssociation.addElement("td").addText(ai.getJoinColumn());
			}
		}
		
		// =====================================================
	    // Field Info
	    // =====================================================
		body.addElement("h4").addText("Field Details");
		Element table = body.addElement("p").addElement("table");
		table.addAttribute("cellpadding", "3");
		table.addAttribute("cellspacing", "3");
		table.addAttribute("border", "1");
		table.addAttribute("style", "border-collapse:collapse;border-color:#999999;font-size:10pt;");
		Element thtr = table.addElement("tr");
		thtr.addAttribute("bgColor", "#a4db42");
		thtr.addElement("th").addText("̸");
		thtr.addElement("th").addText("ʵ ");
		thtr.addElement("th").addText("ʵŸ");
		thtr.addElement("th").addText("÷");
		thtr.addElement("th").addText("÷Ÿ");
		thtr.addElement("th").addText("");
		thtr.addElement("th").addText("ũ");
		thtr.addElement("th").addText("ε");
		thtr.addElement("th").addText("");
		for(FieldInfo fi : entityInfo.getFinfos()){
			Element tdtrElt = table.addElement("tr");
			tdtrElt.addElement("td").addText(fi.getColumnInfo().getName());
			tdtrElt.addElement("td").addText(fi.getName());
			tdtrElt.addElement("td").addText(fi.getType());
			tdtrElt.addElement("td").addText(fi.getColumnName());
			tdtrElt.addElement("td").addText(fi.getSqlType());
			tdtrElt.addElement("td").addText(fi.getIsNullable());
			tdtrElt.addElement("td").addText(fi.getIsUnique());
			tdtrElt.addElement("td").addText(fi.getIndexName());
			tdtrElt.addElement("td").addText(fi.getColumnInfo().getDescription());
		}
		
		return doc;
	}

	public void makeIndexPage(String targetFolderName, List<Class> entityClasses) {
		File newHTML = new File(targetFolderName, "index.html");
		FileWriter fw = null;
		try {
			newHTML.createNewFile();
			fw = new FileWriter(newHTML);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		
		Document doc = null;
		if(fw != null){
			List<EntityInfo> eil = new ArrayList<EntityInfo>();
			for(Class entityClass : entityClasses){
				eil.add(new EntityInfo(entityClass));
			}
			doc = makeIndex(eil);
		}
		
		try {
			fw.write(doc.toString());
			fw.flush();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private Document makeIndex(List<EntityInfo> eil) {
		Document doc = DocumentFactory.makeDocument();
		
		Element head = doc.getHead();
		head.addElement("title").addText("KOMA Domain Document");
		
	    Element body = doc.getBody();
	    body.addAttribute("style", "font-size:10pt;");
	    
	    body.addElement("h3").addText("Domain Classses");
	    
		Element ul = body.addElement("p").addElement("ul");
		for (EntityInfo ei : eil) {
			Element link = ul.addElement("li").addElement("a");
			link.addText(ei.getEntityName());
		    link.addAttribute("href", ei.getEntityName() + ".html");
		}
		
		return doc;
	}
		
		

}
